==> LiteHost-1.2-sources.jar: com/litehost/core/Request.java <==
package com.litehost.core;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

public class Request {
    private String method;
    private String version = "HTTP/1.1";
    private String path;
    private String url;
    private Map<String, String> headers;
    private String body;
    private boolean mutable;

    Request() {
        this.method = null;
        this.url = null;
        this.path = null;
        this.body = null;
        this.headers = new HashMap<>();
        this.mutable = false;
    }

    public static Builder builder() {
        return new Builder();
    }

    public boolean change(String fieldName, Object value) {
        if (!this.mutable)
            return false;
        try {
            Field field = this.getClass().getDeclaredField(fieldName);
            Class<?> fieldClass = field.getType();
            if (!fieldClass.isInstance(value))
                throw new IllegalArgumentException("Incompatible types!");
            field.setAccessible(true);
            field.set(this, value);
            field.setAccessible(false);
            return true;
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
        }
        return false;
    }

    public static class Builder {
        private Request rawRequest;

        Builder() {
            this.rawRequest = new Request();
        }

        public Builder mutable(boolean mutable) {
            this.rawRequest.mutable = mutable;
            return this;
        }

        public Builder version(String version) {
            this.rawRequest.version = version; //todo change to enum in future versions
            return this;
        }


        public Builder URL(String url) {
            int start = url.indexOf('/') + 2;
            int end = url.indexOf('/', start);
            this.rawRequest.url = url;
            this.rawRequest.path = url.substring(end);
            this.rawRequest.headers.put("Host", url.substring(start, end));
            return this;
        }

        public Builder GET() {
            this.rawRequest.method = "GET";
            return this;
        }

        public Builder POST() {
            this.rawRequest.method = "POST";
            return this;
        }

        public Builder PUT() {
            this.rawRequest.method = "PUT";
            return this;
        }

        public Builder DELETE() {
            this.rawRequest.method = "DELETE";
            return this;
        }

        public Builder header(String header, String value) {
            if (this.rawRequest.headers != null)
                this.rawRequest.headers.put(header, value);
            else this.rawRequest.headers = new HashMap<>() {{
                put(header, value);
            }};
            return this;
        }

        public Builder headers(Map<String, String> headers) {
            this.rawRequest.headers = headers;
            String connectionHeader = headers.get("Connection");
            if (connectionHeader == null)
                this.rawRequest.headers.put("Connection", "Close");
            return this;
        }

        public Builder body(String body) {
            this.rawRequest.body = body;
            return this;
        }

        public Request build() {
            return this.rawRequest;
        }

    }

    public String getUrl() {
        return this.url;
    }


    @Override
    public String toString() {
        StringBuilder result = new StringBuilder();
        result.append(this.method)
                .append(' ')
                .append(this.path)
                .append(' ')
                .append(this.version)
                .append('\n');
        if (this.headers != null)
            this.headers.forEach((key, value) -> {
                result.append(key)
                        .append(": ")
                        .append(value)
                        .append('\n');
            });
        result.append('\n');
        if (this.body != null)
            result.append(this.body);
        return result.toString();
    }

}

==> LiteHost-1.2.2-sources.jar: com/litehost/core/Request.java <==
package com.litehost.core;

import java.lang.reflect.Field;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

public class Request {
    private String method;
    private String version = "HTTP/1.1";
    private String path;
    private URI url;
    private Map<String, String> headers;
    private String body;
    private boolean mutable;

    Request() {
        this.method = null;
        this.url = null;
        this.path = null;
        this.body = null;
        this.headers = new HashMap<>();
        this.mutable = false;
    }

    public static Builder builder() {
        return new Builder();
    }

    public boolean change(String fieldName, Object value) {
        if (!this.mutable)
            return false;
        try {
            Field field = this.getClass().getDeclaredField(fieldName);
            Class<?> fieldClass = field.getType();
            if (!fieldClass.isInstance(value))
                throw new IllegalArgumentException("Incompatible types!");
            field.setAccessible(true);
            field.set(this, value);
            field.setAccessible(false);
            return true;
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
        }
        return false;
    }

    public static class Builder {
        private Request rawRequest;

        Builder() {
            this.rawRequest = new Request();
        }

        public Builder mutable(boolean mutable) {
            this.rawRequest.mutable = mutable;
            return this;
        }

        public Builder version(String version) {
            this.rawRequest.version = version; //todo change to enum in future versions
            return this;
        }


        public Builder URL(String url) {
            URI uri = URI.create(url);
            this.rawRequest.url = uri;
            this.rawRequest.path = uri.getPath();
            this.header("Host", uri.getHost());
            return this;
        }

        public Builder GET() {
            this.rawRequest.method = "GET";
            return this;
        }

        public Builder POST() {
            this.rawRequest.method = "POST";
            return this;
        }

        public Builder PUT() {
            this.rawRequest.method = "PUT";
            return this;
        }

        public Builder DELETE() {
            this.rawRequest.method = "DELETE";
            return this;
        }

        public Builder PATCH() {
            this.rawRequest.method = "PATCH";
            return this;
        }

        public Builder header(String header, String value) {
            if (this.rawRequest.headers != null)
                this.rawRequest.headers.put(header, value);
            else this.rawRequest.headers = new HashMap<>() {{
                put(header, value);
            }};
            return this;
        }

        public Builder headers(Map<String, String> headers) {
            this.rawRequest.headers = headers;
            return this;
        }

        public Builder body(String body) {
            this.rawRequest.body = body;
            return this;
        }

        public Request build() {
            if (this.rawRequest.method == null || this.rawRequest.url == null)
                throw new IllegalStateException("Missing method or URL!");
            String connectionHeader = this.rawRequest.headers.put("Connection", "Close");
            if (connectionHeader != null)
                this.rawRequest.headers.put("Connection", connectionHeader);
            return this.rawRequest;
        }

    }

    public String getUrl() {
        return this.url.toString();
    }

    String getAddress() {
        return this.headers.get("Host");
    }

    @Override
    public String toString() {
        StringBuilder result = new StringBuilder();
        result.append(this.method)
                .append(' ')
                .append(this.path)
                .append(' ')
                .append(this.version)
                .append('\n');
        if (this.headers != null)
            this.headers.forEach((key, value) -> {
                result.append(key)
                        .append(": ")
                        .append(value)
                        .append('\n');
            });
        result.append('\n');
        if (this.body != null)
            result.append(this.body);
        return result.toString();
    }

}
